@@ -8,6 +8,8 @@
 namespace netgen
 {
 
+
+
   int CSGeometry :: changeval = 0;
 
 
@@ -81,16 +83,10 @@ namespace netgen
     for (int i = 0; i < solids.Size(); i++)
       if(!to_delete.Contains(solids[i]))
 	to_delete.Append(solids[i]);
+
     for(int i = 0; i < to_delete.Size(); i++)
       delete to_delete[i];    
     
-    /*
-    for (int i = 0; i < solids.Size(); i++)
-      delete solids[i]->S1();
-    for (int i = 0; i < solids.Size(); i++)
-      delete solids[i];
-    */
-
     solids.DeleteAll ();
 
     for (int i = 0; i < splinecurves2d.Size(); i++)
@@ -106,7 +102,7 @@ namespace netgen
       delete delete_them[i];
     delete_them.DeleteAll();
     surfaces.DeleteAll();
-  
+
     for (int i = 0; i < toplevelobjects.Size(); i++)
       delete toplevelobjects[i];
     toplevelobjects.DeleteAll ();
@@ -132,19 +128,21 @@ namespace netgen
   }
 
 
-  extern int CSGGenerateMesh (CSGeometry & geom,
-			      Mesh *& mesh, int perfstepsstart, int perfstepsend,
-			      const char * optstr);
+  extern int CSGGenerateMesh (CSGeometry & geom, 
+			      Mesh *& mesh, MeshingParameters & mparam,
+			      int perfstepsstart, int perfstepsend);
 
-  int CSGeometry :: GenerateMesh (Mesh*& mesh,
-				 int perfstepsstart, int perfstepsend, char* optstring)
+
+  int CSGeometry :: GenerateMesh (Mesh*& mesh, MeshingParameters & mparam,
+				 int perfstepsstart, int perfstepsend)
   {
-    return CSGGenerateMesh (*this, mesh, perfstepsstart, perfstepsend, optstring);
+    return CSGGenerateMesh (*this, mesh, mparam, perfstepsstart, perfstepsend);
   }
 
 
   const Refinement & CSGeometry :: GetRefinement () const
   {
+    // cout << "get CSGeometry - Refinement" << endl;
     // should become class variables
     RefinementSurfaces * ref = new RefinementSurfaces(*this);
     ref -> Set2dOptimizer(new MeshOptimize2dSurfaces(*this));
@@ -182,9 +180,13 @@ namespace netgen
   }
 
 
-
+  void CSGeometry :: Save (string filename) const
+  {
+    fstream ost (filename.c_str());
+    Save (ost);
+  }
   
-  void CSGeometry :: Save (ostream & ost) 
+  void CSGeometry :: Save (ostream & ost) const
   {
     ost << "boundingbox "
 	<< boundingbox.PMin()(0) << " "
@@ -326,7 +328,7 @@ namespace netgen
 
 
 
-  void CSGeometry :: SaveSurfaces (ostream & out)
+  void CSGeometry :: SaveSurfaces (ostream & out) const
   {
     if(singfaces.Size() > 0 || singedges.Size() > 0 || singpoints.Size() > 0)
       {
@@ -442,6 +444,13 @@ namespace netgen
 	  }
 
 
+	else if(classname == "torus")
+	  {
+	    Torus * torus = new Torus(dummypoint,dummyvec,dummydouble, dummydouble);
+	    torus->SetPrimitiveData(coeffs);
+	    AddSurface(torus);
+	    delete_them.Append(torus);
+	  }
 
 
 	else if(classname == "cone")
@@ -483,6 +492,10 @@ namespace netgen
   }
     
 
+  void CSGeometry :: SaveToMeshFile (ostream & ost) const
+  {
+    SaveSurfaces (ost);
+  }
 
 
 
@@ -812,28 +825,20 @@ namespace netgen
     for (int i = 0; i < nsurf; i++)
       isidenticto[i] = i;
   
-    //(*testout) << "jetzt!" << endl;
     for (int i = 0; i < nsurf; i++)
       for (int j = i+1; j < nsurf; j++)
 	{
-	  //(*testout) << "surf" << i << " surf" << j << endl;
 	  if (GetSurface(j) -> IsIdentic (*GetSurface(i), inv, eps))
 	    {
 	      INDEX_2 i2(i, j);
 	      identicsurfaces.Set (i2, inv);
 	      isidenticto[j] = isidenticto[i];
-	      //(*testout) << "surfaces " << i2 << " are identic" << endl;
 	    }
 	}
 
     (*testout) << "identicmap:" << endl;
     for (int i = 0; i < isidenticto.Size(); i++)
       (*testout) << i << " -> " << isidenticto[i] << endl;
-
-    /*    
-    for (int i = 0; i < nsurf; i++)
-      GetSurface(i)->Print (*testout);
-    */
   }
   
 
@@ -846,9 +851,9 @@ namespace netgen
     ReducePrimitiveIterator rpi(box);
     UnReducePrimitiveIterator urpi;
 
-    ((Solid*)sol) -> IterateSolid (rpi);
+    const_cast<Solid*> (sol) -> IterateSolid (rpi);
     sol -> GetSurfaceIndices (locsurf);
-    ((Solid*)sol) -> IterateSolid (urpi);
+    const_cast<Solid*> (sol) -> IterateSolid (urpi);
 
     for (int i = locsurf.Size()-1; i >= 0; i--)
       {
@@ -925,7 +930,7 @@ namespace netgen
     */
   }
 
-
+  /*
   void CSGeometry ::
   GetIndependentSurfaceIndices (const Solid * sol, 
 				const Point<3> & p, Vec<3> & v,
@@ -938,7 +943,7 @@ namespace netgen
     box.CalcDiamCenter();
     GetIndependentSurfaceIndices (sol, box, locsurf);
   }
-
+  */
 
   void CSGeometry ::
   GetIndependentSurfaceIndices (Array<int> & locsurf) const
@@ -969,8 +974,7 @@ namespace netgen
 
 
   void CSGeometry :: 
-  CalcTriangleApproximation(const Box<3> & aboundingbox,
-			    double detail, double facets)
+  CalcTriangleApproximation(double detail, double facets)
   {
     PrintMessage (1, "Calc Triangle Approximation");
 
@@ -1011,7 +1015,7 @@ namespace netgen
 	      continue;
 
 	    TriangleApproximation tas;
-	    GetSurface (j) -> GetTriangleApproximation (tas, aboundingbox, facets);
+	    GetSurface (j) -> GetTriangleApproximation (tas, boundingbox, facets);
 
 	    int oldnp = tams -> GetNP();
 
@@ -1454,7 +1458,7 @@ namespace netgen
 
 
   void CSGeometry :: 
-  IterateAllSolids (SolidIterator & it, bool only_once)
+  IterateAllSolids (SolidIterator & it, bool only_once) const
   {
     if (only_once)
       {
@@ -1479,4 +1483,80 @@ namespace netgen
 		 boundingbox.PMin()(2));
     return max2 (maxs, -mins) * 1.1;
   }
+
+
+
+  class CSGeometryRegister : public GeometryRegister
+  {
+  public:
+    virtual NetgenGeometry * Load (string filename) const;
+    virtual NetgenGeometry * LoadFromMeshFile (istream & ist) const;
+    // virtual VisualScene * GetVisualScene (const NetgenGeometry * geom) const;
+  };
+
+  extern CSGeometry * ParseCSG (istream & istr);
+
+  NetgenGeometry *  CSGeometryRegister :: Load (string filename) const
+  {
+    const char * cfilename = filename.c_str();
+    if (strcmp (&cfilename[strlen(cfilename)-3], "geo") == 0)
+      {
+	PrintMessage (1, "Load CSG geometry file ", cfilename);
+
+	ifstream infile(cfilename);
+
+	CSGeometry * hgeom = ParseCSG (infile);
+	if (!hgeom)
+	  throw NgException ("geo-file should start with 'algebraic3d'");
+
+	hgeom -> FindIdenticSurfaces(1e-8 * hgeom->MaxSize()); 
+	return hgeom;
+      }
+
+    if (strcmp (&cfilename[strlen(cfilename)-3], "ngg") == 0)
+      {
+	PrintMessage (1, "Load new CSG geometry file ", cfilename);
+
+	ifstream infile(cfilename);
+	CSGeometry * hgeom = new CSGeometry("");
+	hgeom -> Load (infile);
+
+	return hgeom;
+      }
+
+
+    
+    return NULL;
+  }
+
+  NetgenGeometry * CSGeometryRegister :: LoadFromMeshFile (istream & ist) const 
+  {
+    string auxstring;
+    if (ist.good())
+      {
+	ist >> auxstring;
+	if (auxstring == "csgsurfaces")
+	  {
+	    CSGeometry * geometry = new CSGeometry ("");
+	    geometry -> LoadSurfaces(ist);
+	    return geometry;
+	  }
+	// else
+	// ist.putback (auxstring);
+      }
+    return NULL;
+  }
+
+
+  class CSGInit
+  {
+  public:
+    CSGInit()
+    {
+      geometryregister.Append (new CSGeometryRegister);
+    }
+  };
+
+  CSGInit csginit;
+
 }
