@@ -188,6 +188,14 @@ public:
 };
 
 
+inline INDEX_2 Sort (const INDEX_2 & i2)
+{
+  INDEX_2 tmp = i2;
+  tmp.Sort();
+  return tmp;
+}
+
+
 ///
 class INDEX_3
 {
@@ -370,6 +378,15 @@ public:
 };
 
 
+inline bool operator< (const INDEX_4 & a, const INDEX_4 & b)
+{
+  for (int j = 0; j < 4; j++)
+    {
+      if (a[j] < b[j]) return true;
+      if (a[j] > b[j]) return false;
+    }
+  return false;
+}
 
 
 
