@@ -50,7 +50,7 @@ namespace netgen
     double sum = 0;
     for (int i = 1; i <= n; i++)
       {
-	double t = (i-0.5)*dt;
+	// double t = (i-0.5)*dt;
 	double fun = hi[i-1];
 	sum += dt / fun;
       }
@@ -249,7 +249,6 @@ namespace netgen
 	  double useh = h;
 	  if (hl > 0) useh = min2 (h, hl);
 	  if (hr > 0) useh = min2 (h, hr);
-
 	  Partition(GetSpline(i), mp, useh, elto0, mesh2d, searchtree, i+1);	    
 	}
       else
