@@ -9,13 +9,15 @@
 /* *************************************************************************/
 
 
+namespace netgen
+{
 
 /**
   Alternating Digital Tree
  */
 
-#include "../include/mystdlib.h"
-#include "../include/myadt.hpp"
+// #include "../include/mystdlib.h"
+// #include "../include/myadt.hpp"
 
 class ADTreeNode
 {
@@ -447,12 +449,12 @@ class Point3dTree
   ADTree3 * tree;
 
 public:
-  Point3dTree (const Point<3> & pmin, const Point<3> & pmax);
-  ~Point3dTree ();
-  void Insert (const Point<3> & p, int pi);
+  DLL_HEADER Point3dTree (const Point<3> & pmin, const Point<3> & pmax);
+  DLL_HEADER ~Point3dTree ();
+  DLL_HEADER void Insert (const Point<3> & p, int pi);
   void DeleteElement (int pi) 
     { tree->DeleteElement(pi); }
-  void GetIntersecting (const Point<3> & pmin, const Point<3> & pmax, 
+  DLL_HEADER void GetIntersecting (const Point<3> & pmin, const Point<3> & pmax, 
 			Array<int> & pis) const;
   const ADTree3 & Tree() const { return *tree; };
 };
@@ -464,6 +466,7 @@ class Box3dTree
   ADTree6 * tree;
   Point<3> boxpmin, boxpmax;
 public:
+  Box3dTree (const Box<3> & abox);
   Box3dTree (const Point<3> & apmin, const Point<3> & apmax);
   ~Box3dTree ();
   void Insert (const Point<3> & bmin, const Point<3> & bmax, int pi);
@@ -478,4 +481,7 @@ public:
 
   const ADTree6 & Tree() const { return *tree; };
 };
+
+}
+
 #endif
