@@ -44,7 +44,7 @@ public:
   // void SetHighOrder (int aorder) { order=aorder; }
   void SetIsHighOrder (bool ho) { ishighorder = ho; }
   
-  void BuildCurvedElements(Refinement * ref, int aorder, bool arational = false);
+  void BuildCurvedElements(const Refinement * ref, int aorder, bool arational = false);
 
   int GetOrder () { return order; }
 
@@ -52,6 +52,7 @@ public:
   bool IsSegmentCurved (SegmentIndex segnr) const;
   bool IsSurfaceElementCurved (SurfaceElementIndex sei) const;
   bool IsElementCurved (ElementIndex ei) const;
+  bool IsElementHighOrder (ElementIndex ei) const;
 
 
   void CalcSegmentTransformation (double xi, SegmentIndex segnr,
@@ -119,17 +120,23 @@ public:
 					    Array<Point<3> > * x,
 					    Array<Vec<3> > * dxdxi);
 
+  template <int DIM_SPACE>
+  void CalcMultiPointSegmentTransformation (SegmentIndex elnr, int n,
+                                            const double * xi, size_t sxi,
+                                            double * x, size_t sx,
+                                            double * dxdxi, size_t sdxdxi);
+
+
   void CalcMultiPointSurfaceTransformation (Array< Point<2> > * xi, SurfaceElementIndex elnr,
 					    Array< Point<3> > * x,
 					    Array< Mat<3,2> > * dxdxi);
-  
+
   template <int DIM_SPACE>
   void CalcMultiPointSurfaceTransformation (SurfaceElementIndex elnr, int n,
                                             const double * xi, size_t sxi,
                                             double * x, size_t sx,
                                             double * dxdxi, size_t sdxdxi);
 
-
   void CalcMultiPointElementTransformation (Array< Point<3> > * xi, ElementIndex elnr,
 					    Array< Point<3> > * x,
 					    Array< Mat<3,3> > * dxdxi);
