@@ -160,8 +160,9 @@ void MeshOptimize3d :: CombineImprove (Mesh & mesh,
 	  for (int k = 0; k < hasonepi.Size(); k++)
 	    {
 	      const Element & elem = mesh[hasonepi[k]];
-	      double err = CalcTetBadness (mesh[elem[0]], mesh[elem[1]],  
-					   mesh[elem[2]], mesh[elem[3]], 0);
+	      double err = CalcBad (mesh.Points(), elem, 0);
+	      // CalcTetBadness (mesh[elem[0]], mesh[elem[1]],  
+	      // mesh[elem[2]], mesh[elem[3]], 0, mparam);
 	      bad2 += err;
 	      oneperr[k] = err;
 	    }
@@ -415,14 +416,8 @@ void MeshOptimize3d :: SplitImprove (Mesh & mesh,
 		  if (el[l] == pi2) has2 = 1;
 		}
 	      if (has1 && has2) 
-		{ // only once
-		  for (l = 0; l < hasbothpoints.Size(); l++)
-		    if (hasbothpoints[l] == elnr)
-		      has1 = 0;
-		  
-		  if (has1)
-		    hasbothpoints.Append (elnr);
-		}
+		if (!hasbothpoints.Contains (elnr))
+		  hasbothpoints.Append (elnr);
 	    }
 	  
 	  bad1 = 0;
@@ -475,7 +470,7 @@ void MeshOptimize3d :: SplitImprove (Mesh & mesh,
 		  }
 	    }
 
-	  PointFunction1 pf (mesh.Points(), locfaces, -1);
+	  PointFunction1 pf (mesh.Points(), locfaces, mp, -1);
 	  OptiParameters par;
 	  par.maxit_linsearch = 50;
 	  par.maxit_bfgs = 20;
@@ -751,7 +746,7 @@ void MeshOptimize3d :: SwapImprove (Mesh & mesh, OPTIMIZEGOAL goal,
 		}
 	      
 	      pi5 = 0;
-	      for (int k = 1; k < 3; k++)
+	      for (int k = 0; k < 3; k++)   // JS, 201212
 		{
 		  const Element & elemk = mesh[hasbothpoints[k]];
 		  bool has1 = 0;
@@ -904,7 +899,7 @@ void MeshOptimize3d :: SwapImprove (Mesh & mesh, OPTIMIZEGOAL goal,
 		}
 	      
 	      pi5 = 0;
-	      for (int k = 1; k < 4; k++)
+	      for (int k = 0; k < 4; k++)
 		{
 		  const Element & elem = mesh[hasbothpoints[k]];
 		  bool has1 = 0;
@@ -920,7 +915,7 @@ void MeshOptimize3d :: SwapImprove (Mesh & mesh, OPTIMIZEGOAL goal,
 		}
 	      
 	      pi6 = 0;
-	      for (int k = 1; k < 4; k++)
+	      for (int k = 0; k < 4; k++)
 		{
 		  const Element & elem = mesh[hasbothpoints[k]];
 		  bool has1 = 0;
@@ -934,7 +929,8 @@ void MeshOptimize3d :: SwapImprove (Mesh & mesh, OPTIMIZEGOAL goal,
 			  pi6 = elem[l];
 		    }
 		}
-	      
+
+
 	      /*
 	      INDEX_2 i22(pi3, pi5);
 	      i22.Sort();
@@ -1148,7 +1144,7 @@ void MeshOptimize3d :: SwapImprove (Mesh & mesh, OPTIMIZEGOAL goal,
 		  mesh[hasbothpoints[1]] = el2;
 		  mesh[hasbothpoints[2]] = el3;
 		  mesh[hasbothpoints[3]] = el4;
-		  
+
 		  for (int k = 0; k < 4; k++)
 		    for (int l = 0; l < 4; l++)
 		      elementsonnode.Add (mesh[hasbothpoints[k]][l], hasbothpoints[k]);
@@ -1181,7 +1177,6 @@ void MeshOptimize3d :: SwapImprove (Mesh & mesh, OPTIMIZEGOAL goal,
 		  mesh[hasbothpoints[2]] = el3b;
 		  mesh[hasbothpoints[3]] = el4b;
 
-
 		  for (int k = 0; k < 4; k++)
 		    for (int l = 0; l < 4; l++)
 		      elementsonnode.Add (mesh[hasbothpoints[k]][l], hasbothpoints[k]);
