@@ -7,24 +7,23 @@ namespace netgen
 
 
   // global variable for visualization
+//   static Array<Point3d> locpoints;
+//   static Array<int> legalpoints;
+//   static Array<Point2d> plainpoints;
+//   static Array<int> plainzones;
+//   static Array<INDEX_2> loclines;
+//   // static int geomtrig;
+//   //static const char * rname;
+//   static int cntelem, trials, nfaces;
+//   static int oldnl;
+//   static int qualclass;
 
-  static Array<Point3d> locpoints;
-  static Array<int> legalpoints;
-  static Array<Point2d> plainpoints;
-  static Array<int> plainzones;
-  static Array<INDEX_2> loclines;
-  // static int geomtrig;
-  //static const char * rname;
-  static int cntelem, trials, nfaces;
-  static int oldnl;
-  static int qualclass;
- 
 
-  Meshing2 :: Meshing2 (const Box<3> & aboundingbox)
+  Meshing2 :: Meshing2 (const MeshingParameters & mp, const Box<3> & aboundingbox)
   {
     boundingbox = aboundingbox;
-
-    LoadRules (NULL);
+    
+    LoadRules (NULL, mp.quad);
     // LoadRules ("rules/quad.rls");
     // LoadRules ("rules/triangle.rls");
 
@@ -73,8 +72,8 @@ namespace netgen
     canuse = 0;
     ruleused = 0;
 
-    cntelem = 0;
-    trials = 0;
+    // cntelem = 0;
+    // trials = 0;
   }
 
   void Meshing2 :: EndMesh ()
@@ -102,8 +101,8 @@ namespace netgen
   }
 
   // should be class variables !!(?)
-  static Vec3d ex, ey;
-  static Point3d globp1;
+  // static Vec3d ex, ey;
+  // static Point3d globp1;
 
   void Meshing2 :: DefineTransformation (const Point3d & p1, const Point3d & p2,
 					 const PointGeomInfo * geominfo1,
@@ -194,15 +193,22 @@ namespace netgen
 
 
 
-  MESHING2_RESULT Meshing2 :: GenerateMesh (Mesh & mesh, double gh, int facenr)
+  MESHING2_RESULT Meshing2 :: GenerateMesh (Mesh & mesh, const MeshingParameters & mp, double gh, int facenr)
   {
     static int timer = NgProfiler::CreateTimer ("surface meshing");
 
     static int timer1 = NgProfiler::CreateTimer ("surface meshing1");
     static int timer2 = NgProfiler::CreateTimer ("surface meshing2");
     static int timer3 = NgProfiler::CreateTimer ("surface meshing3");
+
+    static int ts1 = NgProfiler::CreateTimer ("surface meshing start 1");
+    static int ts2 = NgProfiler::CreateTimer ("surface meshing start 2");
+    static int ts3 = NgProfiler::CreateTimer ("surface meshing start 3");
+
+
     NgProfiler::RegionTimer reg (timer);
 
+    NgProfiler::StartTimer (ts1);
 
     Array<int> pindex, lindex;
     Array<int> delpoints, dellines;
@@ -215,7 +221,6 @@ namespace netgen
     int z1, z2, oldnp(-1);
     bool found;
     int rulenr(-1);
-    int globind;
     Point<3> p1, p2;
 
     const PointGeomInfo * blgeominfo1;
@@ -227,6 +232,17 @@ namespace netgen
     double h, his, hshould;
 
 
+    Array<Point3d> locpoints;
+    Array<int> legalpoints;
+    Array<Point2d> plainpoints;
+    Array<int> plainzones;
+    Array<INDEX_2> loclines;
+    int cntelem = 0, trials = 0, nfaces = 0;
+    int oldnl = 0;
+    int qualclass;
+
+
+
     // test for 3d overlaps
     Box3dTree surfeltree (boundingbox.PMin(),
 			  boundingbox.PMax());
@@ -257,7 +273,6 @@ namespace netgen
     trigsonnode = 0;
     illegalpoint = 0;
   
-
     double totalarea = Area ();
     double meshedarea = 0;
 
@@ -292,13 +307,15 @@ namespace netgen
 	box.Set ( mesh[sel[0]] );
 	box.Add ( mesh[sel[1]] );
 	box.Add ( mesh[sel[2]] );
-	surfeltree.Insert (box, i);
+	surfeltree.Insert (box, seia[i]);
       }
 
-
-
+    NgProfiler::StopTimer (ts1);
+    NgProfiler::StartTimer (ts2);
 
     if (totalarea > 0 || maxarea > 0)
+      meshedarea = mesh.SurfaceArea();
+      /*
       for (SurfaceElementIndex sei = 0; sei < mesh.GetNSE(); sei++)
 	{
 	  const Element2d & sel = mesh[sei];
@@ -315,9 +332,12 @@ namespace netgen
 				      mesh.Point (sel.PNum(4)))).Length() / 2;;
 	  meshedarea += trigarea;
 	}
+      */
+      // cout << "meshedarea = " << meshedarea << " =?= " 
+      // << mesh.SurfaceArea() << endl;
 
-
-
+    NgProfiler::StopTimer (ts2);
+    NgProfiler::StartTimer (ts3);
 
     const char * savetask = multithread.task;
     multithread.task = "Surface meshing";
@@ -329,12 +349,12 @@ namespace netgen
 
     double meshedarea_before = meshedarea;
 
+    NgProfiler::StopTimer (ts3);
 
     while (!adfront ->Empty() && !multithread.terminate)
       {
 	NgProfiler::RegionTimer reg1 (timer1);
 
-
 	if (multithread.terminate)
 	  throw NgException ("Meshing stopped");
 
@@ -419,7 +439,7 @@ namespace netgen
 
 	//(*testout) << "locpoints " << locpoints << endl;
 
-	if (qualclass > mparam.giveuptol2d)
+	if (qualclass > mp.giveuptol2d)
 	  {
 	    PrintMessage (3, "give up with qualclass ", qualclass);
 	    PrintMessage (3, "number of frontlines = ", adfront->GetNFL());
@@ -464,7 +484,6 @@ namespace netgen
 	if (debugparam.haltlargequalclass && qualclass > 50)
 	  debugflag = 1;
 
-
 	// problem recognition !
 	if (found && 
 	    (gpi1 < illegalpoint.Size()+PointIndex::BASE) && 
@@ -492,7 +511,11 @@ namespace netgen
 
 	    // (*testout) << endl;
 
-	    //	    (*testout) << "3d->2d transformation" << endl;
+	    if (debugflag)
+	      {
+		*testout << "3d->2d transformation" << endl;
+		*testout << "3d points: " << endl << locpoints << endl;
+	      }
 
 	    for (int i = 1; i <= locpoints.Size(); i++)
 	      {
@@ -507,6 +530,10 @@ namespace netgen
 	    //	    (*testout) << endl << endl << endl;
 
 
+	    if (debugflag)
+	      *testout << "2d points: " << endl << plainpoints << endl;
+
+
 	    p12d = plainpoints.Get(1);
 	    p22d = plainpoints.Get(2);
 
@@ -704,7 +731,7 @@ namespace netgen
 
 
 
-	    if (mparam.checkchartboundary)
+	    if (mp.checkchartboundary)
 	      {
 		for (int i = 1; i <= chartboundpoints.Size(); i++)
 		  {
@@ -741,7 +768,8 @@ namespace netgen
 	  {
 	    rulenr = ApplyRules (plainpoints, legalpoints, maxlegalpoint,
 				 loclines, maxlegalline, locelements,
-				 dellines, qualclass);
+				 dellines, qualclass, mp);
+
 	    //	    (*testout) << "Rule Nr = " << rulenr << endl;
 	    if (!rulenr)
 	      {
@@ -961,7 +989,7 @@ namespace netgen
 	  }
 
 
-	if (found && mparam.checkoverlap)
+	if (found && mp.checkoverlap)
 	  {
 	    // cout << "checkoverlap" << endl;
 	    // test for overlaps
@@ -1061,17 +1089,14 @@ namespace netgen
 	      {
 		const Point3d & p = critpoints.Get(i);
 		 
+		for (int jj = 0; jj < intersecttrias.Size(); jj++)
+		  {
+		    // int j = intersecttrias.Get(jj);
+		    // const Element2d & el = mesh.SurfaceElement(j);
+
+		    SurfaceElementIndex j = intersecttrias[jj];
+		    const Element2d & el = mesh[j];
 
-		/*
-		  for (j = 1; j <= mesh.GetNSE(); j++)
-		  {
-		*/
-		int jj;
-		for (jj = 1; jj <= intersecttrias.Size(); jj++)
-		  {
-		    int j = intersecttrias.Get(jj);
-		    const Element2d & el = mesh.SurfaceElement(j);
-		  
 		    int ntrig = (el.GetNP() == 3) ? 1 : 2;
 
 		    int jl;
@@ -1217,7 +1242,7 @@ namespace netgen
 	      
 	    for (int i = oldnp+1; i <= locpoints.Size(); i++)
 	      {
-		globind = mesh.AddPoint (locpoints.Get(i));
+		PointIndex globind = mesh.AddPoint (locpoints.Get(i));
 		pindex.Elem(i) = adfront -> AddPoint (locpoints.Get(i), globind);
 	      }
 	      
@@ -1289,7 +1314,7 @@ namespace netgen
 		box.Set (mesh[mtri[0]]);
 		box.Add (mesh[mtri[1]]);
 		box.Add (mesh[mtri[2]]);
-		surfeltree.Insert (box, mesh.GetNSE());
+		surfeltree.Insert (box, mesh.GetNSE()-1);
 
 		const Point3d & sep1 = mesh.Point (mtri.PNum(1));
 		const Point3d & sep2 = mesh.Point (mtri.PNum(2));
@@ -1529,7 +1554,7 @@ namespace netgen
 
 
 // #define OPENGL
-#ifdef OPENGL
+#ifdef OPENGLxx
 
 /* *********************** Draw Surface Meshing **************** */
 
