@@ -394,7 +394,7 @@ namespace netgen
 
 	selbuf.Add (dest, sei);
 	selbuf.Add (dest, sel.GetIndex());
-	selbuf.Add (dest, 0);
+	// selbuf.Add (dest, 0);
 	selbuf.Add (dest, sel.GetNP());
 	
 	for ( int ii = 1; ii <= sel.GetNP(); ii++)
@@ -636,7 +636,7 @@ namespace netgen
 	{
 	  int globsel = selbuf[ii++];
 	  int faceind = selbuf[ii++];
-	  bool isghost = selbuf[ii++];
+	  //bool isghost = selbuf[ii++];
 	  int nep = selbuf[ii++];
 	  Element2d tri(nep);
 	  tri.SetIndex(faceind);
@@ -788,8 +788,8 @@ namespace netgen
       {
 	eptr.Append (eind.Size());
 	const Segment & el = LineSegment(i+1);
-	eind.Append (el[0]);
-	eind.Append (el[1]);
+	eind.Append (el[0]-1);
+	eind.Append (el[1]-1);
       }
     eptr.Append (eind.Size());
     Array<idx_t> epart(ne), npart(nn);
@@ -816,7 +816,114 @@ namespace netgen
       SurfaceElement(i+1).SetPartition(epart[i+GetNE()] + 1);
     for (int i = 0; i < GetNSeg(); i++)
       LineSegment(i+1).SetPartition(epart[i+GetNE()+GetNSE()] + 1);
+
+
+
+    if (GetDimension() == 3)
+      {
+	
+	// surface elements attached to volume elements
+	Array<bool, PointIndex::BASE> boundarypoints (GetNP());
+	boundarypoints = false;
+	
+	for (SurfaceElementIndex sei = 0; sei < GetNSE(); sei++)
+	  {
+	    const Element2d & el = (*this)[sei];
+	    for (int j = 0; j < el.GetNP(); j++)
+	      boundarypoints[el[j]] = true;
+	  }
+	// Build Pnt2Element table, boundary points only
+	Array<int, PointIndex::BASE> cnt(GetNP());
+	cnt = 0;
+	for (ElementIndex ei = 0; ei < GetNE(); ei++)
+	  {
+	    const Element & el = (*this)[ei];
+	    for (int j = 0; j < el.GetNP(); j++)
+	      if (boundarypoints[el[j]])
+		cnt[el[j]]++;
+	  }
+	TABLE<ElementIndex, PointIndex::BASE> pnt2el(cnt);
+	cnt = 0;
+	for (ElementIndex ei = 0; ei < GetNE(); ei++)
+	  {
+	    const Element & el = (*this)[ei];
+	    for (int j = 0; j < el.GetNP(); j++)
+	      if (boundarypoints[el[j]])
+		pnt2el.Add (el[j], ei);
+	  }
+	
+	for (SurfaceElementIndex sei = 0; sei < GetNSE(); sei++)
+	  {
+	    Element2d & sel = (*this)[sei];
+	    PointIndex pi1 = sel[0];
+	    FlatArray<ElementIndex> els = pnt2el[pi1];
+	    
+	    sel.SetPartition (-1);
+	    
+	    for (int j = 0; j < els.Size(); j++)
+	      {
+		const Element & el = (*this)[els[j]];
+		
+		bool hasall = true;
+		
+		for (int k = 0; k < sel.GetNP(); k++)
+		  {
+		    bool haspi = false;
+		    for (int l = 0; l < el.GetNP(); l++)
+		      if (sel[k] == el[l])
+			haspi = true;
+
+		    if (!haspi) hasall = false;
+		  }
+		
+		if (hasall)
+		  {
+		    sel.SetPartition (el.GetPartition());
+		    break;
+		  }
+	      }
+	    if (sel.GetPartition() == -1)
+	      cerr << "no volume element found" << endl;
+	  }
+
+
+	for (SegmentIndex si = 0; si < GetNSeg(); si++)
+	  {
+	    Segment & sel = (*this)[si];
+	    PointIndex pi1 = sel[0];
+	    FlatArray<ElementIndex> els = pnt2el[pi1];
+	    
+	    sel.SetPartition (-1);
+	    
+	    for (int j = 0; j < els.Size(); j++)
+	      {
+		const Element & el = (*this)[els[j]];
+		
+		bool haspi[9] = { false };  // max surfnp
+		
+		for (int k = 0; k < 2; k++)
+		  for (int l = 0; l < el.GetNP(); l++)
+		    if (sel[k] == el[l])
+		      haspi[k] = true;
+		
+		bool hasall = true;
+		for (int k = 0; k < sel.GetNP(); k++)
+		  if (!haspi[k]) hasall = false;
+		
+		if (hasall)
+		  {
+		    sel.SetPartition (el.GetPartition());
+		    break;
+		  }
+	      }
+	    if (sel.GetPartition() == -1)
+	      cerr << "no volume element found" << endl;
+	  }
+	
+	
+      }
   }
+
 #endif
 
 
@@ -1211,7 +1318,7 @@ namespace netgen
     idxtype  *xadj, * adjacency, *v_weights = NULL, *e_weights = NULL;
 
     int weightflag = 0;
-    int numflag = 0;
+    // int numflag = 0;
     int nparts = ntasks - 1;
 
     int options[5];
@@ -1378,11 +1485,9 @@ namespace netgen
     idxtype *v_weights = NULL, *e_weights = NULL;
 
     int weightflag = 0;
-    int numflag = 0;
+    // int numflag = 0;
     int nparts = ntasks - 1;
 
-    int options[5];
-    options[0] = 0;
     int edgecut;
     Array<idxtype> part(ne);
 
@@ -1390,6 +1495,8 @@ namespace netgen
       BubbleSort (adjacency.Range (xadj[el], xadj[el+1]));
 
 #ifdef METIS4	
+    int options[5];
+    options[0] = 0;
     METIS_PartGraphKway ( &ne, &xadj[0], &adjacency[0], v_weights, e_weights, &weightflag, 
 			  &numflag, &nparts, options, &edgecut, &part[0] );
 #else
