@@ -6,89 +6,10 @@
 namespace netgen
 {
 
-  static const MeshOptimize2d * meshthis;
-
-
-#ifdef OLD
-
-  void CalcTriangleBadness (double x2, double x3, double y3, double metricweight,
-				   double h, double & badness, double & g1x, double & g1y)
-  {
-    // badness = sqrt(3.0) /36 * circumference^2 / area - 1 
-    // p1 = (0, 0), p2 = (x2, 0), p3 = (x3, y3);
-
-    Vec2d v23;
-    double l12, l13, l23, cir, area;
-    static const double c = sqrt(3.0) / 36;
-    double c1, c2, c3, c4;
-
-    v23.X() = x3 - x2;
-    v23.Y() = y3;
-
-    l12 = x2;
-    l13 = sqrt (x3*x3 + y3*y3);
-    l23 = v23.Length();
-
-    cir = l12 + l13 + l23;
-    area = 0.5 * x2 * y3;
-
-    if (area <= 1e-24 * cir * cir)
-      {
-	g1x = 0;
-	g1y = 0;
-	badness = 1e10;
-	return;
-      }
-
-    badness = c * cir * cir / area - 1;
-
-    c1 = 2 * c * cir / area;
-    c2 = 0.5 * c * cir * cir / (area * area);
-
-    g1x = c1 * ( - 1 - x3 / l13) - c2 * (-v23.Y());
-    g1y = c1 * (     - y3 / l13) - c2 * ( v23.X());
-  
-    //  metricweight = 0.1;
-    if (metricweight > 0)
-      {
-	// area = (x2 - x1) * (y3 - y1) - (x3 - x1) * (y2 - y1);
-	// add:  metricweight * (area / h^2 + h^2 / area - 2)
-      
-	const double area = x2 * y3;
-	const double dareax1 = -y3; 
-	const double dareay1 = x3 - x2; 
-
-	const double areahh = area / (h * h);
-	const double fac = metricweight * (areahh - 1 / areahh) / area;
-
-	badness += metricweight * (areahh + 1 / areahh - 2);
-	g1x += fac * dareax1;
-	g1y += fac * dareay1; 
-      
-	/*
-	// add: metricweight * (l1^2/h^2 + l2^2/h^2 + l3^2/h2 + h^2/l1^2 + h^2/l2^2 + h^2/l3^2 - 6)
-	double h2 = h*h;
-	double l1 = x2*x2;
-	double l2 = x3*x3+y3*y3;
-	double l3 = (x2-x3)*(x2-x3)+y3*y3;
-	double dl1dx = 2*(-x2);
-	double dl1dy = 0;
-	double dl2dx = -2*x3;
-	double dl2dy = -2*y3;
-
-	badness += (l1/h2 + l2/h2 + l3/h2 +h2/l1 + h2/l2 + h2/l3-6) * metricweight;
-
-	g1x += metricweight * (dl1dx/h2-h2/(l1*l1)*dl1dx + dl2dx/h2-h2/(l2*l2)*dl2dx);
-	g1y += metricweight * (dl1dy/h2-h2/(l1*l1)*dl1dy + dl2dy/h2-h2/(l2*l2)*dl2dy);
-	*/
-      }
-  }
-
-#endif
-
   static const double c_trig = 0.14433756;      // sqrt(3.0) / 12
   static const double c_trig4 = 0.57735026;     // sqrt(3.0) / 3
 
+
   inline double CalcTriangleBadness (double x2, double x3, double y3, 
 				     double metricweight, double h)
   {
@@ -112,7 +33,6 @@ namespace netgen
       }
     return badness;
   }
-
   
   inline void CalcTriangleBadness (double x2, double x3, double y3, double metricweight,
 				   double h, double & badness, double & g1x, double & g1y)
@@ -160,60 +80,20 @@ namespace netgen
 
 
 
-
-
-
-
-
-
-#ifdef OLD
-  double CalcTriangleBadness (const Point3d & p1, 
-			      const Point3d & p2, 
-			      const Point3d & p3,
-			      double metricweight,
-			      double h)
-  {
-    double badness;
-    double g1x, g1y;
-  
-    Vec3d e1 (p1, p2);
-    Vec3d e2 (p1, p3);
-  
-    double e1l = e1.Length() + 1e-24;
-    e1 /= e1l;
-    double e1e2 = (e1 * e2);
-    e2.Add (-e1e2, e1);
-    double e2l = e2.Length();
-  
-    CalcTriangleBadness ( e1l, e1e2, e2l,
-			  metricweight, h, badness, g1x, g1y);
-    return badness;
-  }
-#endif
-
-
-
-
-  double CalcTriangleBadness (const Point3d & p1, 
-			      const Point3d & p2, 
-			      const Point3d & p3,
+  double CalcTriangleBadness (const Point<3> & p1, 
+			      const Point<3> & p2, 
+			      const Point<3> & p3,
 			      double metricweight,
 			      double h)
   {
     // badness = sqrt(3.0) / 12 * (\sum l_i^2) / area - 1 
-    // p1 = (0, 0), p2 = (x2, 0), p3 = (x3, y3);
 
-    Vec3d e12(p1,p2);
-    Vec3d e13(p1,p3);
-    Vec3d e23(p2,p3);
-  
-    double l12_2 = e12.Length2();
-    double l13_2 = e13.Length2();
-    double l23_2 = e23.Length2();
+    Vec<3> e12 = p2-p1; 
+    Vec<3> e13 = p3-p1;
+    Vec<3> e23 = p3-p2;
 
-    double cir_2 = l12_2 + l13_2 + l23_2;
-    Vec3d area_v = Cross (e12, e13);
-    double area = 0.5 * area_v.Length();
+    double cir_2 = e12.Length2() + e13.Length2() + e23.Length2();
+    double area = 0.5 * Cross (e12, e13).Length();
 
     if (area <= 1e-24 * cir_2)
       return 1e10;
@@ -222,29 +102,73 @@ namespace netgen
 
     if (metricweight > 0)
       {
-	// area = (x2 - x1) * (y3 - y1) - (x3 - x1) * (y2 - y1);
 	// add:  metricweight * (area / h^2 + h^2 / area - 2)
-
-	const double areahh = area / (h * h);
+        area *= 2;   // optimum for (2 area) is h^2
+        double areahh = area / (h * h);
 	badness += metricweight * (areahh + 1 / areahh - 2);
       }
 
     return badness;
   }
 
+  double CalcTriangleBadnessGrad (const Point<3> & p1, 
+                                  const Point<3> & p2, 
+                                  const Point<3> & p3,
+                                  Vec<3> & gradp1,
+                                  double metricweight,
+                                  double h)
+  {
+    // badness = sqrt(3.0) / 12 * (\sum l_i^2) / area - 1 
 
-  double CalcTriangleBadness (const Point3d & p1, 
-			      const Point3d & p2, 
-			      const Point3d & p3,
-			      const Vec3d & n,
+    Vec<3> e12 = p2-p1; 
+    Vec<3> e13 = p3-p1;
+    Vec<3> e23 = p3-p2;
+
+    double cir_2 = e12.Length2() + e13.Length2() + e23.Length2();
+    Vec<3> varea = Cross(e12, e13);
+    double area = 0.5 * varea.Length();
+
+    Vec<3> dcir_2 = (-2) * (e12+e13);
+    Vec<3> darea = (0.25/area) * Cross (p2-p3, varea);
+
+    if (area <= 1e-24 * cir_2)
+      {
+        gradp1 = 0;
+        return 1e10;
+      }
+
+    double badness = c_trig * cir_2 / area - 1;
+    gradp1 = c_trig * (1.0/area * dcir_2 - cir_2 / (area*area) * darea);
+
+    if (metricweight > 0)
+      {
+	// add:  metricweight * (area / h^2 + h^2 / area - 2)
+        area *= 2;   // optimum for (2 area) is h^2
+
+        double areahh = area / (h * h);
+	badness += metricweight * (areahh + 1 / areahh - 2);
+
+        gradp1 += (2*metricweight * (1/(h*h) - (h*h)/(area*area))) * darea;
+      }
+
+    return badness;
+  }
+
+
+
+
+  double CalcTriangleBadness (const Point<3> & p1, 
+			      const Point<3> & p2, 
+			      const Point<3> & p3,
+			      const Vec<3> & n,
 			      double metricweight,
 			      double h)
   {
-    Vec3d v1 (p1, p2);
-    Vec3d v2 (p1, p3);
+    Vec<3> v1 = p2-p1;
+    Vec<3> v2 = p3-p1;
 
-    Vec3d e1 = v1;
-    Vec3d e2 = v2;
+    Vec<3> e1 = v1;
+    Vec<3> e2 = v2;
 
     e1 -= (e1 * n) * n;
     e1 /= (e1.Length() + 1e-24);
@@ -255,68 +179,202 @@ namespace netgen
   }
 
 
-
-
-
-  static MeshPoint sp1; 
-  static PointGeomInfo gi1;
-  static Vec<3> normal, t1, t2;
-  static Array<SurfaceElementIndex> locelements(0);
-  static Array<int> locrots(0);
-  static Array<double> lochs(0);
+  class Opti2dLocalData
+  {
+  public:
+    const MeshOptimize2d * meshthis;
+    MeshPoint sp1; 
+    PointGeomInfo gi1;
+    Vec<3> normal, t1, t2;
+    Array<SurfaceElementIndex> locelements;
+    Array<int> locrots;
+    Array<double> lochs;
+    Array<Point<3> > loc_pnts2, loc_pnts3;
   // static int locerr2;
-  static double locmetricweight = 0;
-  static double loch;
-  static int surfi, surfi2;
-  static int uselocalh;
+    double locmetricweight;
+    double loch;
+    int surfi, surfi2;
+    int uselocalh;
+  public:
+    Opti2dLocalData ()
+    {
+      locmetricweight = 0;
+    }
+  };
 
 
   class Opti2SurfaceMinFunction : public MinFunction
   {
     const Mesh & mesh;
+    Opti2dLocalData & ld;
   public:
-    Opti2SurfaceMinFunction (const Mesh & amesh)
-      : mesh(amesh)
+    Opti2SurfaceMinFunction (const Mesh & amesh,
+			     Opti2dLocalData & ald)
+      : mesh(amesh), ld(ald)
     { } ;
-    virtual double FuncGrad (const Vector & x, Vector & g) const;
-    virtual double FuncDeriv (const Vector & x, const Vector & dir, double & deriv) const;
-    virtual double Func (const Vector & x) const;
+
+
+    virtual double Func (const Vector & x) const
+    {
+      Vec<3> n;
+      
+      double badness = 0;
+      
+      ld.meshthis -> GetNormalVector (ld.surfi, ld.sp1, ld.gi1, n);
+      Point<3> pp1 = ld.sp1 + x(0) * ld.t1 + x(1) * ld.t2;
+      
+      for (int j = 0; j < ld.locelements.Size(); j++)
+        {
+          Vec<3> e1 = ld.loc_pnts2[j] - pp1;
+          Vec<3> e2 = ld.loc_pnts3[j] - pp1;
+          
+          if (ld.uselocalh) ld.loch = ld.lochs[j];
+          
+          if (Determinant(e1, e2, n) > 1e-8 * ld.loch * ld.loch)
+            {
+              badness += CalcTriangleBadness (pp1, ld.loc_pnts2[j], ld.loc_pnts3[j],
+                                              ld.locmetricweight, ld.loch);
+            }
+          else
+            {
+              badness += 1e8;
+            }
+        }
+      
+      return badness;
+    }
+
+
+    virtual double FuncGrad (const Vector & x, Vector & g) const
+    {
+      Vec<3> vgrad;
+      Point<3> pp1;
+      
+      vgrad = 0;
+      double badness = 0;
+      
+      pp1 = ld.sp1 + x(0) * ld.t1 + x(1) * ld.t2;
+      
+      for (int j = 0; j < ld.locelements.Size(); j++)
+        {
+          Vec<3> e1 = ld.loc_pnts2[j] - pp1;
+          Vec<3> e2 = ld.loc_pnts3[j] - pp1;
+          
+          if (ld.uselocalh) ld.loch = ld.lochs[j];
+          
+          if (Determinant(e1, e2, ld.normal) > 1e-8 * ld.loch * ld.loch)
+            {
+              Vec<3> hgrad;
+              badness += 
+                CalcTriangleBadnessGrad (pp1, ld.loc_pnts2[j], ld.loc_pnts3[j], hgrad,
+                                         ld.locmetricweight, ld.loch);
+              vgrad += hgrad;
+            }
+          else
+            {
+              badness += 1e8;
+            }
+        }
+      g(0) = ld.t1 * vgrad;
+      g(1) = ld.t2 * vgrad;
+      return badness;
+    }
+
+    virtual double FuncDeriv (const Vector & x, const Vector & dir, double & deriv) const
+    {
+      deriv = 0;
+      double badness = 0;
+      
+      Point<3> pp1 = ld.sp1 + x(0) * ld.t1 + x(1) * ld.t2;
+      Vec<3> dir3d = dir(0) * ld.t1 + dir(1) * ld.t2;
+      
+      for (int j = 0; j < ld.locelements.Size(); j++)
+        {
+          Vec<3> e1 = ld.loc_pnts2[j] - pp1;
+          Vec<3> e2 = ld.loc_pnts3[j] - pp1;
+          
+          if (ld.uselocalh) ld.loch = ld.lochs[j];
+          
+          if (Determinant(e1, e2, ld.normal) > 1e-8 * ld.loch * ld.loch)
+            {
+              Vec<3> hgrad;
+              badness += 
+                CalcTriangleBadnessGrad (pp1, ld.loc_pnts2[j], ld.loc_pnts3[j], hgrad,
+                                         ld.locmetricweight, ld.loch);
+              deriv += dir3d * hgrad;
+            }
+          else
+            {
+              badness += 1e8;
+            }
+        }
+      
+      // cout << "deriv = " << deriv << " =?= ";
+      return badness;
+      /*
+      static int timer = NgProfiler::CreateTimer ("opti2surface - deriv");
+      NgProfiler::RegionTimer reg (timer);
+
+      double eps = 1e-6;
+      Vector xr(2), xl(2);
+      xr = x; xl = x;
+      for (int i = 0; i < 2; i++)
+        {
+          xr(i) = x(i) + eps * dir(i);
+          xl(i) = x(i) - eps * dir(i);
+        }
+      deriv = (Func (xr) - Func(xl) ) / (2*eps); 
+      cout << deriv << endl;
+      return Func(x);
+      */
+    }
+
+
+
+    virtual double XXFuncGrad (const Vector & x, Vector & g) const;
+    virtual double XXFuncDeriv (const Vector & x, const Vector & dir, double & deriv) const;
+
   };
+
   
+  /*
   double Opti2SurfaceMinFunction :: 
   Func (const Vector & x) const
   {
+    static int timer = NgProfiler::CreateTimer ("opti2surface - func");
+    NgProfiler::RegionTimer reg (timer);
+
     Vector g(x.Size());
     return FuncGrad (x, g);
   }
-
+  */
 
   double Opti2SurfaceMinFunction :: 
-  FuncGrad (const Vector & x, Vector & grad) const
+  XXFuncGrad (const Vector & x, Vector & grad) const
   {
+    // static int timer = NgProfiler::CreateTimer ("opti2surface - funcgrad");
+    // NgProfiler::RegionTimer reg (timer);
+
     Vec<3> n, vgrad;
     Point<3> pp1;
-    double g1x, g1y;
-    double badness, hbadness;
 
     vgrad = 0;
-    badness = 0;
+    double badness = 0;
 
-    meshthis -> GetNormalVector (surfi, sp1, gi1, n);
-    pp1 = sp1 + x(0) * t1 + x(1) * t2;
+    ld.meshthis -> GetNormalVector (ld.surfi, ld.sp1, ld.gi1, n);
+    pp1 = ld.sp1 + x(0) * ld.t1 + x(1) * ld.t2;
 
     //  meshthis -> ProjectPoint (surfi, pp1);
     // meshthis -> GetNormalVector (surfi, pp1, n);
 
-    for (int j = 0; j < locelements.Size(); j++)
+    for (int j = 0; j < ld.locelements.Size(); j++)
       {
-	int roti = locrots[j];
-	const Element2d & bel = mesh[locelements[j]];
+        double g1x, g1y, hbadness;
 
-	Vec<3> e1 = mesh[bel.PNumMod(roti + 1)] - pp1;
-	Vec<3> e2 = mesh[bel.PNumMod(roti + 2)] - pp1;
+        Vec<3> e1 = ld.loc_pnts2[j] - pp1;
+        Vec<3> e2 = ld.loc_pnts3[j] - pp1;
 
-	if (uselocalh) loch = lochs[j];
+	if (ld.uselocalh) ld.loch = ld.lochs[j];
 
 	double e1l = e1.Length();
 	if (Determinant(e1, e2, n) > 1e-8 * e1l * e2.Length())
@@ -326,54 +384,54 @@ namespace netgen
             e2 -= e1e2 * e1;
 	    double e2l = e2.Length();
 
-	    CalcTriangleBadness ( e1l, e1e2, e2l, locmetricweight, loch,
-				  hbadness, g1x, g1y);
-
+            CalcTriangleBadness ( e1l, e1e2, e2l, ld.locmetricweight, ld.loch,
+                                  hbadness, g1x, g1y);
+            
 	    badness += hbadness;
             vgrad += g1x * e1 + (g1y/e2l) * e2;
 	  }
 	else
 	  {
-	    (*testout) << "very very bad badness" << endl;
+	    // (*testout) << "very very bad badness" << endl;
 	    badness += 1e8;
 	  }
       }
 
-    vgrad -=  (vgrad * n) * n;
-
-    grad(0) = vgrad * t1;
-    grad(1) = vgrad * t2;
+    // vgrad -=  (vgrad * n) * n;
+    grad(0) = vgrad * ld.t1;
+    grad(1) = vgrad * ld.t2;
     return badness;
   }
 
 
-
-
   double Opti2SurfaceMinFunction :: 
-  FuncDeriv (const Vector & x, const Vector & dir, double & deriv) const
+  XXFuncDeriv (const Vector & x, const Vector & dir, double & deriv) const
   {
+    // static int timer = NgProfiler::CreateTimer ("opti2surface - funcderiv");
+    // NgProfiler::RegionTimer reg (timer);
+
     Vec<3> n, vgrad;
     Point<3> pp1;
-    double g1x, g1y;
-    double badness, hbadness;
 
     vgrad = 0;
-    badness = 0;
+    double badness = 0;
 
-    meshthis -> GetNormalVector (surfi, sp1, gi1, n);
+    ld.meshthis -> GetNormalVector (ld.surfi, ld.sp1, ld.gi1, n);
+    pp1 = ld.sp1 + x(0) * ld.t1 + x(1) * ld.t2;
 
-    pp1 = sp1 + x(0) * t1 + x(1) * t2;
-
-    for (int j = 0; j < locelements.Size(); j++)
+    for (int j = 0; j < ld.locelements.Size(); j++)
       {
-	int roti = locrots[j];
-
-	const Element2d & bel = mesh[locelements[j]];
+        double g1x, g1y, hbadness;
 
+        /*
+        int roti = ld.locrots[j];
+        const Element2d & bel = mesh[ld.locelements[j]];
 	Vec<3> e1 = mesh[bel.PNumMod(roti + 1)] - pp1;
 	Vec<3> e2 = mesh[bel.PNumMod(roti + 2)] - pp1;
-
-	if (uselocalh) loch = lochs[j];
+        */
+        Vec<3> e1 = ld.loc_pnts2[j] - pp1;
+        Vec<3> e2 = ld.loc_pnts3[j] - pp1;
+	if (ld.uselocalh) ld.loch = ld.lochs[j];
 
 	double e1l = e1.Length();
 	if (Determinant(e1, e2, n) > 1e-8 * e1l * e2.Length())
@@ -382,8 +440,7 @@ namespace netgen
 	    double e1e2 = e1 * e2;
 	    e2 -= e1e2 * e1;
 	    double e2l = e2.Length();
-
-	    CalcTriangleBadness ( e1l, e1e2, e2l, locmetricweight, loch,
+	    CalcTriangleBadness ( e1l, e1e2, e2l, ld.locmetricweight, ld.loch,
 				  hbadness, g1x, g1y);
 
 	    badness += hbadness;
@@ -391,13 +448,13 @@ namespace netgen
 	  }
 	else
 	  {
-	    (*testout) << "very very bad badness" << endl;
+	    // (*testout) << "very very bad badness" << endl;
 	    badness += 1e8;
 	  }
       }
 
-    vgrad -= (vgrad * n) * n;
-    deriv = dir(0) * (vgrad*t1) + dir(1) * (vgrad*t2);
+    // vgrad -= (vgrad * n) * n;
+    deriv = dir(0) * (vgrad*ld.t1) + dir(1) * (vgrad*ld.t2);
 
     return badness;
   }
@@ -416,9 +473,12 @@ namespace netgen
   class Opti2EdgeMinFunction : public MinFunction
   {
     const Mesh & mesh;
+    Opti2dLocalData & ld;
+
   public:
-    Opti2EdgeMinFunction (const Mesh & amesh)
-      : mesh(amesh) { } ;
+    Opti2EdgeMinFunction (const Mesh & amesh,
+			  Opti2dLocalData & ald)
+      : mesh(amesh), ld(ald) { } ;
 
     virtual double FuncGrad (const Vector & x, Vector & g) const;
     virtual double Func (const Vector & x) const;
@@ -441,13 +501,13 @@ namespace netgen
     vgrad = 0.0;
     badness = 0;
 
-    pp1 = sp1 + x(0) * t1;
-    meshthis -> ProjectPoint2 (surfi, surfi2, pp1);
+    pp1 = ld.sp1 + x(0) * ld.t1;
+    ld.meshthis -> ProjectPoint2 (ld.surfi, ld.surfi2, pp1);
 
-    for (j = 0; j < locelements.Size(); j++)
+    for (j = 0; j < ld.locelements.Size(); j++)
       {
-	rot = locrots[j];
-	const Element2d & bel = mesh[locelements[j]];
+	rot = ld.locrots[j];
+	const Element2d & bel = mesh[ld.locelements[j]];
 
 	v1 = mesh[bel.PNumMod(rot + 1)] - pp1;
 	v2 = mesh[bel.PNumMod(rot + 2)] - pp1;
@@ -458,21 +518,21 @@ namespace netgen
 	e2 -= (e1 * e2) * e1;
 	e2 /= e2.Length();
 
-	if (uselocalh) loch = lochs[j];
-	CalcTriangleBadness ( (e1 * v1), (e1 * v2), (e2 * v2), locmetricweight, loch,
+	if (ld.uselocalh) ld.loch = ld.lochs[j];
+	CalcTriangleBadness ( (e1 * v1), (e1 * v2), (e2 * v2), ld.locmetricweight, ld.loch,
 			      hbadness, g1(0), g1(1));
 
 	badness += hbadness;
         vgrad += g1(0) * e1 + g1(1) * e2;
       }
 
-    meshthis -> GetNormalVector (surfi, pp1, n1);
-    meshthis -> GetNormalVector (surfi2, pp1, n2);
+    ld.meshthis -> GetNormalVector (ld.surfi, pp1, n1);
+    ld.meshthis -> GetNormalVector (ld.surfi2, pp1, n2);
 
     v1 = Cross (n1, n2);
     v1.Normalize();
 
-    grad(0) = (vgrad * v1) * (t1 * v1);
+    grad(0) = (vgrad * v1) * (ld.t1 * v1);
 
     return badness;
   }
@@ -483,9 +543,12 @@ namespace netgen
   class Opti2SurfaceMinFunctionJacobian : public MinFunction
   {
     const Mesh & mesh;
+    Opti2dLocalData & ld;
+
   public:
-    Opti2SurfaceMinFunctionJacobian (const Mesh & amesh)
-      : mesh(amesh)
+    Opti2SurfaceMinFunctionJacobian (const Mesh & amesh,
+				     Opti2dLocalData & ald)
+      : mesh(amesh), ld(ald)
     { } ;
     virtual double FuncGrad (const Vector & x, Vector & g) const;
     virtual double FuncDeriv (const Vector & x, const Vector & dir, double & deriv) const;
@@ -514,9 +577,9 @@ namespace netgen
     vgrad = 0;
     badness = 0;
 
-    meshthis -> GetNormalVector (surfi, sp1, gi1, n);
+    ld.meshthis -> GetNormalVector (ld.surfi, ld.sp1, ld.gi1, n);
 
-    pp1 = sp1 + x(0) * t1 + x(1) * t2;
+    pp1 = ld.sp1 + x(0) * ld.t1 + x(1) * ld.t2;
 
     //  meshthis -> ProjectPoint (surfi, pp1);
     //  meshthis -> GetNormalVector (surfi, pp1, n);
@@ -526,19 +589,19 @@ namespace netgen
 
     grad = 0;
 
-    for (int j = 1; j <= locelements.Size(); j++)
+    for (int j = 1; j <= ld.locelements.Size(); j++)
       {
-	lpi = locrots.Get(j);
+	lpi = ld.locrots.Get(j);
 	const Element2d & bel = 
-	  mesh[locelements.Get(j)];
+	  mesh[ld.locelements.Get(j)];
       
 	gpi = bel.PNum(lpi);
 
 	for (int k = 1; k <= bel.GetNP(); k++)
 	  {
 	    PointIndex pi = bel.PNum(k);
-	    pts2d.Elem(pi) = Point2d (t1 * (mesh.Point(pi) - sp1), 
-				      t2 * (mesh.Point(pi) - sp1)); 
+	    pts2d.Elem(pi) = Point2d (ld.t1 * (mesh.Point(pi) - ld.sp1), 
+				      ld.t2 * (mesh.Point(pi) - ld.sp1)); 
 	  }				    
 	pts2d.Elem(gpi) = Point2d (x(0), x(1));
       
@@ -586,30 +649,30 @@ namespace netgen
     vgrad = 0;
     badness = 0;
 
-    meshthis -> GetNormalVector (surfi, sp1, gi1, n);
+    ld.meshthis -> GetNormalVector (ld.surfi, ld.sp1, ld.gi1, n);
 
     // pp1 = sp1;
     //    pp1.Add2 (x.Get(1), t1, x.Get(2), t2);
-    pp1 = sp1 + x(0) * t1 + x(1) * t2;
+    pp1 = ld.sp1 + x(0) * ld.t1 + x(1) * ld.t2;
 
     static Array<Point2d> pts2d;
     pts2d.SetSize(mesh.GetNP());
 
     deriv = 0;
 
-    for (j = 1; j <= locelements.Size(); j++)
+    for (j = 1; j <= ld.locelements.Size(); j++)
       {
-	lpi = locrots.Get(j);
+	lpi = ld.locrots.Get(j);
 	const Element2d & bel = 
-	  mesh[locelements.Get(j)];
+	  mesh[ld.locelements.Get(j)];
       
 	gpi = bel.PNum(lpi);
 
 	for (k = 1; k <= bel.GetNP(); k++)
 	  {
 	    PointIndex pi = bel.PNum(k);
-	    pts2d.Elem(pi) = Point2d (t1 * (mesh.Point(pi) - sp1), 
-				      t2 * (mesh.Point(pi) - sp1)); 
+	    pts2d.Elem(pi) = Point2d (ld.t1 * (mesh.Point(pi) - ld.sp1), 
+				      ld.t2 * (mesh.Point(pi) - ld.sp1)); 
 	  }				    
 	pts2d.Elem(gpi) = Point2d (x(0), x(1));
       
@@ -650,7 +713,7 @@ namespace netgen
     ;
   }
 
-  void MeshOptimize2d :: ImproveMesh (Mesh & mesh)
+  void MeshOptimize2d :: ImproveMesh (Mesh & mesh, const MeshingParameters & mp)
   {
     if (!faceindex)
       {
@@ -658,24 +721,25 @@ namespace netgen
 
 	for (faceindex = 1; faceindex <= mesh.GetNFD(); faceindex++)
 	  {
-	    ImproveMesh (mesh);
+	    ImproveMesh (mesh, mp);
 	    if (multithread.terminate)
 	      throw NgException ("Meshing stopped");
 	  }
 	faceindex = 0;
 	return;
       }
- 
 
     static int timer = NgProfiler::CreateTimer ("MeshSmoothing 2D");
     static int timer1 = NgProfiler::CreateTimer ("MeshSmoothing 2D start");
+    static int timer2 = NgProfiler::CreateTimer ("MeshSmoothing 2D - BFGS");
 
     NgProfiler::RegionTimer reg (timer);
     NgProfiler::StartTimer (timer1);
 
     CheckMeshApproximation (mesh);
 
-    // SurfaceElementIndex sei;
+    Opti2dLocalData ld;
+
 
     Array<SurfaceElementIndex> seia;
     mesh.GetSurfaceElementsOfFace (faceindex, seia);
@@ -688,23 +752,48 @@ namespace netgen
 	  break;
 	}
 
-    int loci;
-    double fact;
-    int moveisok;
-
-    PointGeomInfo ngi;
-    Point3d origp;
-
-
-
-    Vec3d n1, n2;
-    Vector x(2), xedge(1);
+    Vector x(2);
 
     Array<MeshPoint, PointIndex::BASE> savepoints(mesh.GetNP());
 
-    uselocalh = mparam.uselocalh;
+    ld.uselocalh = mp.uselocalh;
 
+    Array<int, PointIndex::BASE> compress(mesh.GetNP());
+    Array<PointIndex> icompress;
+    for (int i = 0; i < seia.Size(); i++)
+      {
+	const Element2d & el = mesh[seia[i]];
+	for (int j = 0; j < el.GetNP(); j++)
+	  compress[el[j]] = -1;
+      }
+    for (int i = 0; i < seia.Size(); i++)
+      {
+	const Element2d & el = mesh[seia[i]];
+	for (int j = 0; j < el.GetNP(); j++)
+	  if (compress[el[j]] == -1)
+	    {
+	      compress[el[j]] = icompress.Size();
+	      icompress.Append(el[j]);
+	    }
+      }
+    Array<int> cnta(icompress.Size());
+    cnta = 0;
+    for (int i = 0; i < seia.Size(); i++)
+      {
+	const Element2d & el = mesh[seia[i]];
+	for (int j = 0; j < el.GetNP(); j++)
+	  cnta[compress[el[j]]]++;
+      }
+    TABLE<SurfaceElementIndex> elementsonpoint(cnta);
+    for (int i = 0; i < seia.Size(); i++)
+      {
+	const Element2d & el = mesh[seia[i]];
+	for (int j = 0; j < el.GetNP(); j++)
+	  elementsonpoint.Add (compress[el[j]], seia[i]);
+      }
 
+    
+    /*
     Array<int, PointIndex::BASE> nelementsonpoint(mesh.GetNP());
     nelementsonpoint = 0;
     for (int i = 0; i < seia.Size(); i++)
@@ -722,15 +811,19 @@ namespace netgen
 	for (int j = 0; j < el.GetNP(); j++)
 	  elementsonpoint.Add (el[j], seia[i]);
       }
+    */
+    
 
 
-    loch = mparam.maxh;
-    locmetricweight = metricweight;
-    meshthis = this;
+    ld.loch = mp.maxh;
+    ld.locmetricweight = metricweight;
+    ld.meshthis = this;
 
-    Opti2SurfaceMinFunction surfminf(mesh);
-    Opti2EdgeMinFunction edgeminf(mesh);
-    Opti2SurfaceMinFunctionJacobian surfminfj(mesh);
+
+
+    Opti2SurfaceMinFunction surfminf(mesh, ld);
+    Opti2EdgeMinFunction edgeminf(mesh, ld);
+    Opti2SurfaceMinFunctionJacobian surfminfj(mesh, ld);
 
     OptiParameters par;
     par.maxit_linsearch = 8;
@@ -738,7 +831,7 @@ namespace netgen
 
     /*
     int i, j, k;
-
+    Vector xedge(1);
       if (improveedges)
       for (i = 1; i <= mesh.GetNP(); i++)
       if (mesh.PointType(i) == EDGEPOINT)
@@ -788,6 +881,7 @@ namespace netgen
 	  
       if (surfi2 && !surfi3)
       {
+      Vec3d n1, n2;
       GetNormalVector (surfi, sp1, n1);
       GetNormalVector (surfi2, sp1, n2);
       t1 = Cross (n1, n2);
@@ -810,128 +904,139 @@ namespace netgen
     if (mesh.GetNP() > 1000)
       {
 	plotchar = '+';
-	modplot = 10;
+	modplot = 100;
       }
     if (mesh.GetNP() > 10000)
       {
 	plotchar = 'o';
-	modplot = 100;
+	modplot = 1000;
+      }
+    if (mesh.GetNP() > 100000)
+      {
+	plotchar = 'O';
+	modplot = 10000;
       }
     int cnt = 0;
 
 
     NgProfiler::StopTimer (timer1);
 
-
+    /*
     for (PointIndex pi = PointIndex::BASE; pi < mesh.GetNP()+PointIndex::BASE; pi++)
       if (mesh[pi].Type() == SURFACEPOINT)
-	{
-	  if (multithread.terminate)
-	    throw NgException ("Meshing stopped");
-	
-	  cnt++;
-	  if (cnt % modplot == 0 && writestatus)
-	    {
-	      printeddot = 1;
-	      PrintDot (plotchar);
-	    }
-
-	  if (elementsonpoint[pi].Size() == 0)
-	    continue;
-
-	
-	  sp1 = mesh[pi];
-
-	  Element2d & hel = mesh[elementsonpoint[pi][0]];
-
-	  int hpi = 0;
-	  for (int j = 1; j <= hel.GetNP(); j++)
-	    if (hel.PNum(j) == pi)
+    */
+    for (int hi = 0; hi < icompress.Size(); hi++)
+      {
+	PointIndex pi = icompress[hi];
+	if (mesh[pi].Type() == SURFACEPOINT)
+	  {
+	    if (multithread.terminate)
+	      throw NgException ("Meshing stopped");
+	    
+	    cnt++;
+	    if (cnt % modplot == 0 && writestatus)
 	      {
-		hpi = j;
-		break;
+		printeddot = 1;
+		PrintDot (plotchar);
 	      }
-
-	  gi1 = hel.GeomInfoPi(hpi);
-	  SelectSurfaceOfPoint (sp1, gi1);
-	  
-	  locelements.SetSize(0);
-	  locrots.SetSize (0);
-	  lochs.SetSize (0);
+	    
+	    // if (elementsonpoint[pi].Size() == 0) continue;
+	    if (elementsonpoint[hi].Size() == 0) continue;
 	
-	  for (int j = 0; j < elementsonpoint[pi].Size(); j++)
-	    {
-	      SurfaceElementIndex sei = elementsonpoint[pi][j];
-	      const Element2d & bel = mesh[sei];
-	      surfi = mesh.GetFaceDescriptor(bel.GetIndex()).SurfNr();
+	    ld.sp1 = mesh[pi];
 	    
-	      locelements.Append (sei);
+	    // Element2d & hel = mesh[elementsonpoint[pi][0]];
+	    Element2d & hel = mesh[elementsonpoint[hi][0]];
 	    
-	      for (int k = 1; k <= bel.GetNP(); k++)
-		if (bel.PNum(k) == pi)
-		  {
-		    locrots.Append (k);
-		    break;
-		  }
-	      
-	      if (uselocalh)
+	    int hpi = 0;
+	    for (int j = 1; j <= hel.GetNP(); j++)
+	      if (hel.PNum(j) == pi)
 		{
-		  Point3d pmid = Center (mesh[bel[0]], mesh[bel[1]], mesh[bel[2]]);
-		  lochs.Append (mesh.GetH(pmid));
+		  hpi = j;
+		  break;
 		}
-	    }
+
+	    ld.gi1 = hel.GeomInfoPi(hpi);
+	    SelectSurfaceOfPoint (ld.sp1, ld.gi1);
 	  
-	  GetNormalVector (surfi, sp1, gi1, normal);
-	  t1 = normal.GetNormal ();
-	  t2 = Cross (normal, t1);
+	    ld.locelements.SetSize(0);
+	    ld.locrots.SetSize (0);
+	    ld.lochs.SetSize (0);
+            ld.loc_pnts2.SetSize (0);
+            ld.loc_pnts3.SetSize (0);
+	
+	    for (int j = 0; j < elementsonpoint[hi].Size(); j++)
+	      {
+		SurfaceElementIndex sei = elementsonpoint[hi][j];
+		const Element2d & bel = mesh[sei];
+		ld.surfi = mesh.GetFaceDescriptor(bel.GetIndex()).SurfNr();
+		
+		ld.locelements.Append (sei);
+		
+		for (int k = 1; k <= bel.GetNP(); k++)
+		  if (bel.PNum(k) == pi)
+		    {
+		      ld.locrots.Append (k);
+                      ld.loc_pnts2.Append (mesh[bel.PNumMod(k + 1)]);
+                      ld.loc_pnts3.Append (mesh[bel.PNumMod(k + 2)]);
+		      break;
+		    }
+		
+		if (ld.uselocalh)
+		  {
+		    Point3d pmid = Center (mesh[bel[0]], mesh[bel[1]], mesh[bel[2]]);
+		    ld.lochs.Append (mesh.GetH(pmid));
+		  }
+	      }
+	    
+	  GetNormalVector (ld.surfi, ld.sp1, ld.gi1, ld.normal);
+	  ld.t1 = ld.normal.GetNormal ();
+	  ld.t2 = Cross (ld.normal, ld.t1);
 	  
 	  // save points, and project to tangential plane
-	  for (int j = 0; j < locelements.Size(); j++)
+	  for (int j = 0; j < ld.locelements.Size(); j++)
 	    {
-	      const Element2d & el = mesh[locelements[j]];
+	      const Element2d & el = mesh[ld.locelements[j]];
 	      for (int k = 0; k < el.GetNP(); k++)
 		savepoints[el[k]] = mesh[el[k]];
 	    }
 
-	  for (int j = 0; j < locelements.Size(); j++)
+	  for (int j = 0; j < ld.locelements.Size(); j++)
 	    {
-	      const Element2d & el = mesh[locelements[j]];
+	      const Element2d & el = mesh[ld.locelements[j]];
 	      for (int k = 0; k < el.GetNP(); k++)
 		{
 		  PointIndex hhpi = el[k];
-		  double lam = normal * (mesh[hhpi] - sp1);
-		  mesh[hhpi] -= lam * normal;
+		  double lam = ld.normal * (mesh[hhpi] - ld.sp1);
+		  mesh[hhpi] -= lam * ld.normal;
 		}
 	    }
 	  
 	  x = 0;
-	  par.typx = lochs[0];
+	  par.typx = 0.3*ld.lochs[0];
+
+          NgProfiler::StartTimer (timer2);
 
 	  if (mixed)
 	    {
-	      //	      (*testout) << "vorher : " << surfminfj.Func (x) << endl;
 	      BFGS (x, surfminfj, par, 1e-6);
-	      //	      (*testout) << "nachher: " << surfminfj.Func (x) << endl;
-	      //	      (*testout) << "x = " << x << endl;
 	    }
 	  else
 	    {
-	      //	      (*testout) << "vorher : " << surfminf.Func (x) << endl;
 	      BFGS (x, surfminf, par, 1e-6);
-	      //	      (*testout) << "nachher: " << surfminf.Func (x) << endl;
-	      //	      (*testout) << "x = " << x << endl;
 	    }
 
-	
-	  origp = mesh[pi];
-	  loci = 1;
-	  fact = 1;
-	  moveisok = 0;
+          NgProfiler::StopTimer (timer2);
+
+	  Point3d origp = mesh[pi];
+	  int loci = 1;
+	  double fact = 1;
+	  int moveisok = 0;
 
 	  // restore other points
-	  for (int j = 0; j < locelements.Size(); j++)
+	  for (int j = 0; j < ld.locelements.Size(); j++)
 	    {
-	      const Element2d & el = mesh[locelements[j]];
+	      const Element2d & el = mesh[ld.locelements[j]];
 	      for (int k = 0; k < el.GetNP(); k++)
 		{
 		  PointIndex hhpi = el[k];
@@ -949,7 +1054,7 @@ namespace netgen
 	      mesh[pi].Y() = origp.Y() + (x.Get(1) * t1.Y() + x.Get(2) * t2.Y())*fact;
 	      mesh[pi].Z() = origp.Z() + (x.Get(1) * t1.Z() + x.Get(2) * t2.Z())*fact;
               */
-              Vec<3> hv = x(0) * t1 + x(1) * t2;
+              Vec<3> hv = x(0) * ld.t1 + x(1) * ld.t2;
               Point3d hnp = origp + Vec3d (hv);
               mesh[pi](0) = hnp.X();
               mesh[pi](1) = hnp.Y();
@@ -960,14 +1065,15 @@ namespace netgen
 	      // ProjectPoint (surfi, mesh[pi]);
 	      // moveisok = CalcPointGeomInfo(surfi, ngi, mesh[pi]); 
 
-	      ngi = gi1;
-	      moveisok = ProjectPointGI (surfi, mesh[pi], ngi);
+	      PointGeomInfo ngi;
+	      ngi = ld.gi1;
+	      moveisok = ProjectPointGI (ld.surfi, mesh[pi], ngi);
 	      // point lies on same chart in stlsurface
 	    
 	      if (moveisok)
 		{
-		  for (int j = 0; j < locelements.Size(); j++)
-		    mesh[locelements[j]].GeomInfoPi(locrots[j]) = ngi;
+		  for (int j = 0; j < ld.locelements.Size(); j++)
+		    mesh[ld.locelements[j]].GeomInfoPi(ld.locrots[j]) = ngi;
 		}
 	      else
 		{
@@ -976,10 +1082,10 @@ namespace netgen
 	    
 	    }
 	}
-
+      }
     if (printeddot)
       PrintDot ('\n');
-  
+
     CheckMeshApproximation (mesh);
     mesh.SetNextTimeStamp();
   }
