@@ -69,6 +69,10 @@
 static char* name = new char[100];
 static int nbe = 0;
 
+#ifdef WIN32
+#define M_PI 3.14159265358979323846
+#endif 
+
 //=======================================================================
 //function : Partition_Loop
 //purpose  : 
