@@ -22,7 +22,7 @@
 #include <BRepAdaptor_Surface.hxx>
 #include <BRepAlgo_AsDes.hxx>
 #include <BRepAlgo_FaceRestrictor.hxx>
-#include <BRepOffset_DataMapOfShapeReal.hxx>
+// #include <BRepOffset_DataMapOfShapeReal.hxx>   // V6.3
 #include <BRepTopAdaptor_FClass2d.hxx>
 #include <BRep_Builder.hxx>
 #include <BRep_Tool.hxx>
@@ -37,6 +37,7 @@
 #include <TopTools_DataMapIteratorOfDataMapOfShapeListOfShape.hxx>
 #include <TopTools_DataMapIteratorOfDataMapOfShapeShape.hxx>
 #include <TopTools_DataMapOfShapeInteger.hxx>
+#include <TopTools_DataMapOfShapeReal.hxx>    // V6.5
 #include <TopTools_DataMapOfShapeShape.hxx>
 #include <TopTools_IndexedMapOfShape.hxx>
 #include <TopTools_ListIteratorOfListOfShape.hxx>
@@ -519,7 +520,8 @@ static void prepareDegen (const TopoDS_Edge&                        DegEdge,
     DC.Initialize( DegEdge, F );
 
   // avoid intersecting twice the same edge
-  BRepOffset_DataMapOfShapeReal EUMap ( EdgesList.Extent() );
+  // BRepOffset_DataMapOfShapeReal EUMap ( EdgesList.Extent() );  // V6.3
+  TopTools_DataMapOfShapeReal EUMap ( EdgesList.Extent() );   // V6.5
 
   Standard_Real U, f, l;
   BRep_Tool::Range (DegEdge, f, l);
