@@ -15,10 +15,13 @@ namespace netgen
 
 #define DIVIDEEDGESECTIONS 1000
 #define IGNORECURVELENGTH 1e-4
+#define VSMALL 1e-10
 
 
    bool merge_solids = 1;
 
+
+  // can you please explain what you intend to compute here (JS) !!!
    double Line :: Dist (Line l)
    {
       Vec<3> n = p1-p0;
@@ -26,7 +29,7 @@ namespace netgen
       double nq = n*q;
 
       Point<3> p = p0 + 0.5*n;
-      double lambda = (p-l.p0)*n / nq;
+      double lambda = (p-l.p0)*n / (nq + VSMALL);
 
       if (lambda >= 0 && lambda <= 1)
       {
@@ -62,7 +65,7 @@ namespace netgen
       if (mparam.maxh * kappa < 1)
          hret = mparam.maxh;
       else
-         hret = 1 / kappa;
+         hret = 1 / (kappa + VSMALL);
 
       if (mparam.maxh < hret)
          hret = mparam.maxh;
@@ -109,8 +112,8 @@ namespace netgen
 
       gp_Pnt2d parmid;
 
-      parmid.SetX(0.3*(par0.X()+par1.X()+par2.X()));
-      parmid.SetY(0.3*(par0.Y()+par1.Y()+par2.Y()));
+      parmid.SetX( (par0.X()+par1.X()+par2.X()) / 3 );
+      parmid.SetY( (par0.Y()+par1.Y()+par2.Y()) / 3 );
 
       if (depth%3 == 0)
       {
@@ -633,7 +636,7 @@ namespace netgen
          }
 
          (*testout) << "mesh face " << k << endl;
-         multithread.percent = 100 * k / (mesh.GetNFD()+1e-10);
+         multithread.percent = 100 * k / (mesh.GetNFD() + VSMALL);
          geom.facemeshstatus[k-1] = -1;
 
 
@@ -788,7 +791,7 @@ namespace netgen
          MESHING2_RESULT res;
 
          try {
-            res = meshing.GenerateMesh (mesh, maxh, k);
+	   res = meshing.GenerateMesh (mesh, mparam, maxh, k);
          }
 
          catch (SingularMatrixException)
@@ -901,7 +904,7 @@ namespace netgen
          //      if (k != 36) continue;
 
          //      (*testout) << "optimize face " << k << endl;
-         multithread.percent = 100 * k / (mesh.GetNFD()+1e-10);
+         multithread.percent = 100 * k / (mesh.GetNFD() + VSMALL);
 
          FaceDescriptor & fd = mesh.GetFaceDescriptor(k);
 
@@ -933,7 +936,7 @@ namespace netgen
                meshopt.SetWriteStatus (0);
 
                //	    (*testout) << "ImproveMesh (mesh)" << endl;
-               meshopt.ImproveMesh (mesh);
+               meshopt.ImproveMesh (mesh, mparam);
             }
 
             {
@@ -958,7 +961,7 @@ namespace netgen
                meshopt.SetWriteStatus (0);
 
                //	    (*testout) << "ImproveMesh (mesh)" << endl;
-               meshopt.ImproveMesh (mesh);
+               meshopt.ImproveMesh (mesh, mparam);
             }
          }
 
@@ -1000,9 +1003,16 @@ namespace netgen
 
          int nedges = geom.emap.Extent();
 
+		 double mincurvelength = IGNORECURVELENGTH;
          double maxedgelen = 0;
          double minedgelen = 1e99;
 
+		 if(occparam.resthminedgelenenable) 
+		 {
+			mincurvelength = occparam.resthminedgelen;
+			if(mincurvelength < IGNORECURVELENGTH) mincurvelength = IGNORECURVELENGTH;
+		 }
+
          multithread.task = "Setting local mesh size (elements per edge)";
 
          // setting elements per edge
@@ -1017,7 +1027,7 @@ namespace netgen
             BRepGProp::LinearProperties(e, system);
             double len = system.Mass();
 
-            if (len < IGNORECURVELENGTH)
+            if (len < mincurvelength)
             {
                (*testout) << "ignored" << endl;
                continue;
@@ -1229,7 +1239,7 @@ namespace netgen
                   mindist = min (mindist, line.Dist(lines[num]));
                }
 
-               mindist *= occparam.resthcloseedgefac;
+               mindist /= (occparam.resthcloseedgefac + VSMALL);
 
                if (mindist < 1e-3)
                {
@@ -1263,9 +1273,8 @@ namespace netgen
 
 
 
-   int OCCGenerateMesh (OCCGeometry & geom, Mesh *& mesh,
-                        int perfstepsstart, int perfstepsend,
-                        char * optstr)
+  int OCCGenerateMesh (OCCGeometry & geom, Mesh *& mesh, MeshingParameters & mparam,
+		       int perfstepsstart, int perfstepsend)
    {
       multithread.percent = 0;
 
