@@ -43,6 +43,10 @@ namespace netgen
 	atof (Tcl_GetVar (interp, "::stloptions.resthcloseedgefac", 0));
       occparam.resthcloseedgeenable =
 	atoi (Tcl_GetVar (interp, "::stloptions.resthcloseedgeenable", 0));
+	  occparam.resthminedgelen = 
+	atof (Tcl_GetVar (interp, "::stloptions.resthminedgelen", 0));
+	  occparam.resthminedgelenenable = 
+	atoi (Tcl_GetVar (interp, "::stloptions.resthminedgelenenable", 0));
     }
   };
 
