@@ -858,7 +858,8 @@ proc viewingoptionsdialog { } {
 	label $f.fshrink.lab -text "Shrink elements"
 	scale $f.fshrink.scale -orient horizontal -length 200 -from 0 -to 1.0001 \
 	    -resolution 0.01  -tickinterval 0.25 \
-	    -command { Ng_SetVisParameters; redraw } -variable  viewoptions.shrink
+	    -command { Ng_SetVisParameters; after idle redraw } \
+            -variable  viewoptions.shrink
 
 
 	checkbutton $f.showidentified -text "Show identified points" \
