@@ -7,7 +7,9 @@
 #include "incvis.hpp"
 #include <meshing.hpp>
 
-
+#ifdef NGPYTHON
+#include <boost/python.hpp>
+#endif
 
 namespace netgen
 {
@@ -430,8 +432,7 @@ int Tcl_AppInit(Tcl_Interp * interp)
 #ifdef LINKMKL
 extern "C" double ddot_(int *n, double *dx, int *incx, double *dy, 
                         int *incy);
-
-int mkldummy()
+void mkldummy()
 {
   int n = 1, one = 1;
   double a = 1, b = 1; 
