@@ -336,7 +336,7 @@ namespace netgen
 	if(infile.good())
 	  {
 	    infile >> auxstring;
-	    if(auxstring == "csgsurfaces")
+	    if(geometry && auxstring == "csgsurfaces")
 	      geometry -> LoadSurfaces(infile);
 	  }
       }
@@ -2116,7 +2116,13 @@ namespace netgen
       return TCL_ERROR;
     }
     sprintf(str,"rm %s", filename2);
-    system(str);
+
+    err = system(str);
+    if (err != 0)
+      {
+        Tcl_SetResult (Togl_Interp(togl), (char*)"Cannot delete temporary file", TCL_VOLATILE);
+        return TCL_ERROR;
+      }
 
     return TCL_OK;
   }
@@ -2137,7 +2143,7 @@ namespace netgen
         cout << "Snapshot to file '" << filename << "'" << endl;
 
         int w = Togl_Width (togl);
-        w = int((w + 1) / 4) * 4 + 4;
+        // w = int((w + 1) / 4) * 4 + 4;
         int h = Togl_Height (togl);
 
         // unsigned char * buffer = new unsigned char[w*h*4];
@@ -2148,7 +2154,7 @@ namespace netgen
         struct jpeg_error_mgr jerr;
         FILE *outfile = fopen(filename,"wb");
         JSAMPROW row_pointer[1];
-        int row_stride, quality = 85; // 1...100
+        int row_stride, quality = 100; // 1...100
 
         cinfo.err = jpeg_std_error( &jerr );
         jpeg_create_compress( &cinfo );
@@ -2161,7 +2167,7 @@ namespace netgen
         cinfo.in_color_space = JCS_RGB;
 
         jpeg_set_defaults( &cinfo );
-        jpeg_set_quality( &cinfo, quality, TRUE );
+        jpeg_set_quality( &cinfo, quality, FALSE );   // TRUE
         jpeg_start_compress( &cinfo, TRUE );
 
         row_stride = 3*w;
@@ -2236,12 +2242,12 @@ namespace netgen
     double tmp = context->width * context->height * 255.0 * 255.0;
     double Ypsnr = psnr( context->coded_frame->error[0] / tmp );
     double quality = context->coded_frame->quality/(double)FF_QP2LAMBDA;
-    char pict_type = av_get_pict_type_char(context->coded_frame->pict_type);
+    char pict_type = av_get_picture_type_char(context->coded_frame->pict_type);
     cout << "video: frame=" << count_frames << " type=" << pict_type;
     cout << " size=" << bytes << " PSNR(Y)=" << Ypsnr << " dB q=" << (float)quality << endl;
   }
 
-
+ 
 
   static int Ng_VideoClip (struct Togl * togl,
                            int argc, tcl_const char *argv[])
@@ -2320,18 +2326,23 @@ namespace netgen
 
         // Init codec context etc.:
         //--------------------------
-        context = avcodec_alloc_context();
+        // context = avcodec_alloc_context();
+	context = avcodec_alloc_context3(codec);
 
         context->bit_rate = bitrate;
         context->width = nx;
         context->height = ny;
-	context->time_base = (AVRational){ 1, 25 };
+        AVRational s;
+        s.num = 1;
+        s.den = 25;
+        context->time_base = s;
         context->gop_size = gopsize;
         context->max_b_frames = bframes;
         context->pix_fmt = PIX_FMT_YUV420P;
         context->flags |= CODEC_FLAG_PSNR;
 
-        if( avcodec_open( context, codec ) < 0 ) {
+        // if( avcodec_open( context, codec ) < 0 ) {
+	if( avcodec_open2( context, codec, NULL) < 0 ) {
           cout << "can't open codec" << endl;
           avcodec_close( context );
           av_free( context );
@@ -2759,6 +2770,10 @@ void Ng_SetMouseEventHandler (netgen::MouseEventHandler * handler)
   vsmesh.SetMouseEventHandler (handler);
 }
 
+void Ng_SetUserVisualizationObject (netgen::UserVisualizationObject * vis)
+{
+  vssolution.AddUserVisualizationObject (vis);
+}
 
 void Ng_ClearSolutionData ()
 {
@@ -2870,7 +2885,7 @@ void PlayAnimFile(const char* name, int speed, int maxcnt)
     vispar.shininess = atof (Tcl_GetVar (interp, "::viewoptions.mat.shininess", TCL_GLOBAL_ONLY));
     vispar.locviewer = atoi (Tcl_GetVar (interp, "::viewoptions.light.locviewer", TCL_GLOBAL_ONLY));
     vispar.transp = atof (Tcl_GetVar (interp, "::viewoptions.mat.transp", TCL_GLOBAL_ONLY));
-
+    
     VisualizationParameters::Clipping hclip;
     hclip.normal.X() = atof (Tcl_GetVar (interp, "::viewoptions.clipping.nx", TCL_GLOBAL_ONLY));
     hclip.normal.Y() = atof (Tcl_GetVar (interp, "::viewoptions.clipping.ny", TCL_GLOBAL_ONLY));
@@ -2884,9 +2899,10 @@ void PlayAnimFile(const char* name, int speed, int maxcnt)
       atoi (Tcl_GetVar (interp, "::viewoptions.clipping.notdomain", TCL_GLOBAL_ONLY));
 
     if ( ! (hclip == vispar.clipping) )
-      hclip.timestamp = NextTimeStamp();
-    vispar.clipping = hclip;
-
+      {
+	vispar.clipping = hclip;
+	vispar.clipping.timestamp = NextTimeStamp();
+      }
 
     vispar.whitebackground = atoi (Tcl_GetVar (interp, "::viewoptions.whitebackground", TCL_GLOBAL_ONLY));
     vispar.drawcoordinatecross = atoi (Tcl_GetVar (interp, "::viewoptions.drawcoordinatecross", TCL_GLOBAL_ONLY));
@@ -2941,8 +2957,13 @@ void PlayAnimFile(const char* name, int speed, int maxcnt)
       atoi (Tcl_GetVar (interp, "::viewoptions.drawpyramids", TCL_GLOBAL_ONLY));
     vispar.drawhexes =
       atoi (Tcl_GetVar (interp, "::viewoptions.drawhexes", TCL_GLOBAL_ONLY));
+    /*
     vispar.shrink =
       atof (Tcl_GetVar (interp, "::viewoptions.shrink", TCL_GLOBAL_ONLY));
+    */
+    double hshrink = atof (Tcl_GetVar (interp, "::viewoptions.shrink", TCL_GLOBAL_ONLY));
+    if (hshrink != vispar.shrink)
+      { vispar.shrink = hshrink; vispar.clipping.timestamp = NextTimeStamp();}
     vispar.drawidentified =
       atoi (Tcl_GetVar (interp, "::viewoptions.drawidentified", TCL_GLOBAL_ONLY));
     vispar.drawpointnumbers =
